@@ -1,23 +1,7 @@
-from __future__ import unicode_literals
-from __future__ import division
-from __future__ import absolute_import
-from dali.compat import python_2_unicode_compatible
-import struct
-import sys
+import warnings
 
 
-if sys.version_info < (3,):
-    _integer_types = (int, long)
-else:
-    _integer_types = (int,)
-
-
-_bad_init_data = TypeError(
-    "data must be a sequence of integers all in the range 0..255 or an integer")
-
-
-@python_2_unicode_compatible
-class Frame(object):
+class Frame:
     """A DALI frame.
 
     A Frame consists of one start bit, n data bits, and one stop
@@ -26,7 +10,7 @@ class Frame(object):
     Instances of this object are mutable.
     """
 
-    def __init__(self, bits, data=0):
+    def __init__(self, bits, data=0, new_exceptions=False):
         """Initialise a Frame with the supplied number of data bits.
 
         :parameter bits: the number of data bits in the Frame
@@ -40,17 +24,24 @@ class Frame(object):
             raise ValueError(
                 "Frames must contain at least 1 data bit")
         self._bits = bits
-        if isinstance(data, _integer_types):
+        if isinstance(data, int):
             self._data = data
         else:
-            d = 0
-            for b in data:
-                if not isinstance(b, int):
-                    raise _bad_init_data
-                if b < 0 or b > 255:
-                    raise _bad_init_data
-                d = (d << 8) | b
-            self._data = d
+            if new_exceptions:
+                self._data = int.from_bytes(data, 'big')
+            else:
+                warnings.warn("Frame() will raise ValueError in the future "
+                              "when an invalid initialisation sequence is "
+                              "passed, instead of TypeError. Pass "
+                              "new_exceptions=True to Frame() to use "
+                              "the new behaviour now.",
+                              DeprecationWarning, stacklevel=2)
+                try:
+                    self._data = int.from_bytes(data, 'big')
+                except ValueError:
+                    raise TypeError(
+                        "data must be a sequence of integers all in the "
+                        "range 0..255 or an integer")
         if self._data < 0:
             raise ValueError(
                 "Initial data must not be negative")
@@ -70,13 +61,13 @@ class Frame(object):
     def __eq__(self, other):
         try:
             return self._bits == other._bits and self._data == other._data
-        except:
+        except Exception:
             return False
 
     def __ne__(self, other):
         try:
             return self._bits != other._bits or self._data != other._data
-        except:
+        except Exception:
             return True
 
     def _readslice(self, key):
@@ -133,7 +124,7 @@ class Frame(object):
         """
         if isinstance(key, slice):
             hi, lo = self._readslice(key)
-            if not isinstance(value, _integer_types):
+            if not isinstance(value, int):
                 raise TypeError("value must be an integer")
             if value.bit_length() > (hi + 1 - lo):
                 raise ValueError("value will not fit in supplied slice")
@@ -164,7 +155,7 @@ class Frame(object):
         try:
             return Frame(self._bits + other._bits,
                          self._data << other._bits | other._data)
-        except:
+        except Exception:
             raise TypeError("Frame can only be added to another Frame")
 
     @property
@@ -182,15 +173,7 @@ class Frame(object):
         long, the first element in the sequence contains fewer than 8
         bits.
         """
-        remaining = len(self)
-        l = []
-        d = self._data
-        while remaining > 0:
-            l.append(d & 0xff)
-            d = d >> 8
-            remaining = remaining - 8
-        l.reverse()
-        return l
+        return list(self.pack)
 
     @property
     def pack(self):
@@ -199,8 +182,32 @@ class Frame(object):
         If the frame is not an exact multiple of 8 bits long, the
         first byte in the string will contain fewer than 8 bits.
         """
-        s = self.as_byte_sequence
-        return struct.pack("B" * len(s), *s)
+        return self._data.to_bytes(
+            (len(self) // 8) + (1 if len(self) % 8 else 0),
+            'big')
+
+    def pack_len(self, l, new_exceptions=False):
+        """The contents of the frame represented as a fixed length byte string.
+
+        The least significant bit of the frame is aligned to the end
+        of the byte string.  The start of the byte string is padded with zeroes.
+
+        If the frame will not fit in the byte string, raises
+        ValueError (with new_exceptions=False) or OverflowError (with
+        new_exceptions=True).
+        """
+        if new_exceptions:
+            return self._data.to_bytes(l, 'big')
+        warnings.warn("Frame.pack_len() will raise OverflowError in the "
+                      "future when an invalid length is passed, instead of "
+                      "ValueError. Pass new_exceptions=True to use "
+                      "the new behaviour now.",
+                      DeprecationWarning, stacklevel=2)
+        try:
+            return self._data.to_bytes(l, 'big')
+        except OverflowError:
+            raise ValueError(
+                f"Frame length {len(self)} will not fit in {l} bytes")
 
     def __str__(self):
         return "{}({},{})".format(self.__class__.__name__, len(self),
@@ -240,7 +247,10 @@ class BackwardFrame(Frame):
     """
 
     def __init__(self, data):
-        Frame.__init__(self, 8, data)
+        super().__init__(8, data)
+
+    def __str__(self):
+        return "{}({})".format(self.__class__.__name__, self._data)
 
 
 class BackwardFrameError(BackwardFrame):
@@ -253,5 +263,5 @@ class BackwardFrameError(BackwardFrame):
     """
 
     def __init__(self, data):
-        BackwardFrame.__init__(self, data)
+        super().__init__(data)
         self._error = True
